/* $Id: lsidprotocol.js,v 1.2 2005/04/12 04:46:07 rpage Exp $ */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LSID protocol 0.1.
 *
 * The Initial Developer of the Original Code is
 * Roderic D.M. Page (r.page@bio.gla.ac.uk).
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// components defined in this file
const LSIDPROT_HANDLER_CONTRACTID 	= "@mozilla.org/network/protocol;1?name=lsidres";
const LSIDPROT_HANDLER_CID 			= Components.ID("{5ec001f6-9e77-4d2f-8e86-b454d52212de}");


// components used in this file
const NS_IOSERVICE_CID 				= "{9ac9e770-18bc-11d3-9337-00104ba0fd40}";
const NS_PREFSERVICE_CONTRACTID 	= "@mozilla.org/preferences-service;1";
const URI_CONTRACTID 				= "@mozilla.org/network/simple-uri;1";
const NS_WINDOWWATCHER_CONTRACTID 	= "@mozilla.org/embedcomp/window-watcher;1";
const STREAMIOCHANNEL_CONTRACTID 	= "@mozilla.org/network/stream-io-channel;1";

// interfaces used in this file
const nsIProtocolHandler    		= Components.interfaces.nsIProtocolHandler;
const nsIURI                		= Components.interfaces.nsIURI;
const nsISupports           		= Components.interfaces.nsISupports;
const nsIIOService          		= Components.interfaces.nsIIOService;
const nsIPrefService        		= Components.interfaces.nsIPrefService;
const nsIWindowWatcher      		= Components.interfaces.nsIWindowWatcher;
const nsIChannel            		= Components.interfaces.nsIChannel;


/***** LSIDProtocolHandler *****/

function LSIDProtocolHandler(scheme)
{
    this.scheme = scheme;
}

// attribute defaults
LSIDProtocolHandler.prototype.defaultPort = -1;
LSIDProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;

LSIDProtocolHandler.prototype.allowPort = function(aPort, aScheme)
{
    return false;
}

LSIDProtocolHandler.prototype.newURI = function(aSpec, aCharset, aBaseURI)
{
    var uri = Components.classes[URI_CONTRACTID].createInstance(nsIURI);
    uri.spec = aSpec;
    return uri;
}

LSIDProtocolHandler.prototype.newChannel = function(aURI)
{
	var handle;
    var proxy;
    if (aURI.spec.indexOf("://") == 7){
      handle = aURI.spec.substr("lsidres://".length);
    }
    else {
      handle = aURI.spec.substr("lsidres:".length);
    }
    /* proxy = "http://biopathways.ibm.nebiogrid.org/resolver/"; */
    
    // My resolver that outputs HTML
    proxy = "http://darwin.zoology.gla.ac.uk/cgi-bin/lsid.pl?lsid=";
       
    var ioServ = Components.classesByID[NS_IOSERVICE_CID].getService();
    ioServ = ioServ.QueryInterface(nsIIOService);
    var uri = ioServ.newURI(proxy+handle, null, null);
    var chan = ioServ.newChannelFromURI(uri);
    return chan;

}


/***** LSIDProtocolHandlerFactory *****/

function LSIDProtocolHandlerFactory(scheme)
{
    this.scheme = scheme;
}

LSIDProtocolHandlerFactory.prototype.createInstance = function(outer, iid)
{
    if(outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

    if(!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new LSIDProtocolHandler(this.scheme);
}

var factory_lsid = new LSIDProtocolHandlerFactory("lsidres");

/***** LSIDModule *****/

var LSIDModule = new Object();

LSIDModule.registerSelf = function(compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    // register ed2k protocol handler
    compMgr.registerFactoryLocation(LSIDPROT_HANDLER_CID,
                                    "LSID protocol handler",
                                    LSIDPROT_HANDLER_CONTRACTID,
                                    fileSpec, location, type);

}

LSIDModule.unregisterSelf = function(compMgr, fileSpec, location)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    // unregister our components
    compMgr.unregisterFactoryLocation(LSIDPROT_HANDLER_CID, fileSpec);
}

LSIDModule.getClassObject = function(compMgr, cid, iid)
{
    if(!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if(cid.equals(LSIDPROT_HANDLER_CID)) return factory_lsid;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

LSIDModule.canUnload = function(compMgr)
{
    return true;    // our objects can be unloaded
}

/***** Entrypoint *****/

function NSGetModule(compMgr, fileSpec)
{
    return LSIDModule;
}
